package JSci.maths.fields;

import JSci.maths.*;
import JSci.maths.groups.AbelianGroupMember;

/**
* The IntegerRing class encapsulates the ring of integer numbers.
* @version 1.0
* @author Mark Hale
*/
public final class IntegerRing extends Object implements Ring {
        public final static MathInteger ZERO=new MathInteger(0);
        public final static MathInteger ONE=new MathInteger(1);

        private static IntegerRing _instance;
        /**
        * Constructs a ring of integer numbers.
        */
        private IntegerRing() {}
        /**
        * Constructs a ring of integer numbers.
        * Singleton.
        */
        public static final IntegerRing getInstance() {
                if(_instance == null) {
                        synchronized(IntegerRing.class) {
                                if(_instance == null)
                                        _instance = new IntegerRing();
                        }
                }
                return _instance;
        }
        /**
        * Returns the integer number zero.
        */
        public AbelianGroupMember zero() {
                return ZERO;
        }
        /**
        * Returns true if the integer number is equal to zero.
        */
        public boolean isZero(AbelianGroupMember g) {
                return ZERO.equals(g);
        }
        /**
        * Returns true if one integer number is the negative of the other.
        */
        public boolean isNegative(AbelianGroupMember a,AbelianGroupMember b) {
                return ZERO.equals(a.add(b));
        }
        /**
        * Returns the integer number one.
        */
        public RingMember one() {
                return ONE;
        }
        /**
        * Returns true if the integer number is equal to one.
        */
        public boolean isOne(RingMember r) {
                return ONE.equals(r);
        }
}

